
# q-expansion of newform 3450.2.a.f, downloaded from the LMFDB on 02 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 3450
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-1], [-1], [0], [1], [0], [-2], [-3], [2], [1], [9], [-10], [-11], [12], [10], [9], [0], [-6], [14], [-14], [3], [-11], [8], [9], [3], [10], [-9], [7], [-12], [11], [9], [-8], [12], [-9], [5], [0], [8], [-14], [16], [3], [6], [0], [-1], [-18], [25], [15], [-1], [23], [22], [21], [2], [24], [27], [8], [3], [18], [18], [6], [-28], [-26], [3], [16], [-18], [-5], [-15], [-8], [27], [5], [28], [30], [-10], [6], [36], [-8], [13], [20], [36], [-30], [-8], [-18], [17], [15], [26], [30], [-2], [20], [0], [-6], [-32], [3], [-8], [33], [-18], [28], [-30], [-19], [-36], [-27], [-39], [4], [44], [1], [42], [-24], [-42], [26], [22], [12], [-36], [24], [-19], [-14], [19], [-30], [44], [5], [-21], [-14], [-21], [-9], [15], [-31], [7], [-12], [12], [11], [-6], [-19], [-15], [-32], [-29], [-1], [30], [-13], [13], [-42], [-34], [42], [28], [30], [-6], [-52], [18], [-8], [33], [2], [0], [-44], [6], [-4], [-27], [-38], [30], [-11], [-39], [-38], [-24], [-31], [-42], [-50], [42], [12], [-15], [-2], [-21], [-21], [-36], [-40], [-26], [62], [27], [51], [-10], [-18], [22], [14], [-51], [-19], [6], [19], [-37], [46], [21], [16], [48], [-36], [-18], [16], [4], [-31], [15], [-2], [24], [50], [-15], [0], [39], [-10], [-41], [-45], [33], [30], [-28], [-2], [14], [-36], [-6], [65], [0], [12], [17], [-26], [30], [1], [-3], [-24], [-46], [4], [-12], [24], [42], [50], [-40], [-60], [16], [0], [-16], [-6], [0], [-8], [36], [-14], [62], [32], [-18], [28], [12], [-10], [-66], [18], [-39], [21], [29], [22], [56], [42], [75], [-38], [60], [20], [-72], [22], [-18], [-78], [62], [9], [66], [-10], [-32], [-9], [10], [19], [-3], [-10], [7], [-48], [-64], [48], [33], [-26], [30], [8], [-2], [28], [59], [82], [-50], [-12], [-4], [-40], [27], [-48], [-28], [54], [-19], [-68], [36], [34], [-72], [2], [-36], [-78], [3], [-24], [39], [-20], [0], [8], [-3], [-12], [-53], [58], [-48], [11], [-60], [-40], [-38], [-6], [-13], [36], [-50], [0], [-78], [69], [-5], [-48], [-88], [0], [0], [-44], [54], [-22], [-38], [3], [-86], [33], [38], [2], [19], [-24], [-42], [62], [18], [2], [42], [-58], [66], [74], [-27], [77], [-47], [10], [42], [-30], [-37], [-12], [18], [-52], [19], [36], [-12], [-7], [55], [33], [-11], [-67], [-72], [6], [-21], [12], [24], [55], [-30], [-48], [-87], [28], [94], [78], [79], [-64], [-6], [-73], [21], [-90], [-1], [-20], [-84], [78], [-20], [-27], [25], [-84], [-69], [-56], [18], [-28], [39], [-82], [-50], [-80], [-42], [50], [-6], [18], [97], [78], [40], [-52], [-78], [14], [6], [44], [42], [37], [-30], [87], [-16], [1], [-18], [40], [6], [-14], [-33], [12], [98], [-2], [-51], [-15], [10], [18], [-6], [-6], [-38], [63], [-30], [-23], [-39], [3], [18], [17], [-21]]
